
	;---------------------------------------------------------------------
	section .text
	align 1
	;---------------------------------------------------------------------

	; 3x4 keypad wired to RN3 = PAI PCLK DSACK0 DSACK1 AS DS AVEC

PORTE equ 0FA11h  ; byte
DDRE  equ 0FA15h  ; byte
PEPAR equ 0FA17h  ; byte

PACTL equ 0F90Ch  ; byte

TOC3  equ 0F918h  ; word
TCTL1 equ 0F91Eh  ; byte
TMSK1 equ 0F920h  ; byte
TFLG1 equ 0F922h  ; byte
CFORC equ 0F924h  ; byte

OC3_V equ 008Ch

	global _init_keypad

_init_keypad

	pshm k

	clrb
	tbek          ; EXT RAM

	ldd  # OC3_kp_wait_release
	std  OC3_V

	ldab # 0FFh
	tbek          ; EXT SFR

	clrb
	stab PEPAR    ; all are gpio, bus control not used
	stab PORTE    ; only outputting low, if outputting at all

	ldab # 34h    ; AS DS AVEC outputs;
	stab DDRE     ; pull all columns to detect any key

	bclr TCTL1, # 0Ch ; OM3, OL3 - output pin disconnected
	bclr CFORC, # 20h ; FOC3 - not forced
	bset TMSK1, # 20h ; OC3I - enable int

	pulm k
	rts

	;------------------------  windvane and keypad are mutually exclusive
	global _init_windvane

_init_windvane

	pshm k

	ldab # 0FFh
	tbek          ; EXT SFR

	clrb
	stab PEPAR    ; all are gpio, bus control not used
	stab PORTE    ; only outputting low, if outputting at all
	stab DDRE     ; all pins inputs

	pulm k
	rts

	global _windvane

_windvane

	pshm k

	ldab # 0FFh
	tbek          ; EXT SFR

	ldaa PACTL    ; PCLK 0800h - ld just the MSByte into A
	ldab PORTE    ; 5 of the 6 bits - mask is 37h

	pulm k

	andd # 0837h  ; ? ? ? ? PCLK ? ? ? : ? ? AS DS ? AVEC DSACK1 DSACK0
	adda # 0FFh   ; PCLK into carry
	tba           ; AS and DS into A (0x30), carry kept
	rolb          ; B has (0x0F) AVEC DSACK1 DSACK0 PCLK
	andd # 300Fh  ; mask nybbles
	aba           ; merge
	tab           ; byte return
	clra          ; zero extend
	rts

	;---------------------------------------------------------------------
	section .inthand
	align 1

	; column outputs must be all active, before this test

KP_ROWS	MACRO
	ldab PORTE    ; DSACK0 0001 DSACK1 0002
	ldaa PACTL    ; PCLK 0800 and PAI 8000 - ld just the MSByte
	andd # 8803h
	eord # 8803h  ; EQ if no key
	ENDM

	; TOC3 is not changed, continuation of keypress
	; is tested every 64 msec or so. When no key is pressed,
	; vector is changed and next int will again happen after 64 msec.
	; column outputs stay all active, most of the time.

OC3_kp_wait_release

	pshm d,k
	ldab # 0FFh
	tbek

	bclr TFLG1, # 20h ; OC3F

	KP_ROWS
	bne  L4       ; some keys still pressed

	clrb
	tbek          ; EXT RAM

	stab _keydown

	ldd  # OC3_kp_wait_press
	std  OC3_V
L4
	pulm d,k
	rti

	; Until a key is seen, TOC3 is updated for about 100 Hz rate.
	; Then the key is determined and vector changed to _wait_release.

kp_row
	ldab PORTE    ; DSACK0 0001 DSACK1 0002
	andb # 3
	eorb # 3
	bne  L8       ; 1 or 2 (or 3 - not handled)
	ldab PACTL    ; PCLK 0800 and PAI 8000 - ld just the MSByte
	andb # 88h
	eorb # 88h
	beq  L8       ; neither
	aslb          ; carry if row 4
	ldab # 3
	adcb # 0      ; NE in either case
L8
	rts

OC3_kp_wait_press

	pshm d,k
	ldab # 0FFh
	tbek

	KP_ROWS
	bne  L5            ; key has been pressed

	ldd  # 10000       ; about 10 msec interval
	addd TOC3
	std  TOC3
	bclr TFLG1, # 20h  ; OC3F

	pulm d,k
	rti
L5
	ldab # 20h
	stab DDRE          ; pull single column
	clra
	bsr  kp_row
	bne  L6

	ldab # 10h
	stab DDRE          ; pull single column
	adda # 4
	bsr  kp_row
	bne  L6

	ldab # 04h
	stab DDRE          ; pull single column
	adda # 4
	bsr  kp_row
	bne  L6

	clrd               ; where did it go ?
L6
	aba

	ldab # 34h         ; AS DS AVEC outputs;
	stab DDRE          ; pull all columns to detect any key

	bclr TFLG1, # 20h  ; OC3F

	tsta
	beq  L7            ; glitch case

	clrb
	tbek               ; EXT RAM
	tbxk
	tbyk
	pshm e,x,y         ; _keypad_enq() is nice to Z

	xgab               ; B contained zero
	tde
	ldx  # _keymap - 1 ; adjust for 1...12 index
	ldab e,x
	stab _keydown
	tde
	jsr  _keypad_enq   ; void keypad_enq(byte key);  XXX IPL very high

	pulm e,x,y

	ldd  # OC3_kp_wait_release
	std  OC3_V
L7
	pulm d,k
	rti

	end
