
static byte
chr(byte c)
{
	if (c < ' ' || c >= 0x7F)
		return ('.');

	return (c);
}

static void
print_control(byte c, int cmdresp)
{
	int pf = c & 0x10;
	int nr = c >> 5;
	char *qd;

	qd = (pf == 0)      ? ""
	   : (cmdresp == 2) ? "?"
	   : (cmdresp == 1) ? "."
	   : "p/f";

	if ((c & 1) == 0) {

		printf("I%d%s r%d", (c >> 1) & 7, qd, nr);

	} else if ((c & 2) == 0) {

		switch (c & 0x0F) {
		case 0x01:
			printf("RR%s r%d", qd, nr);
			break;
		case 0x05:
			printf("RNR%s r%d", qd, nr);
			break;
		case 0x09:
			printf("REJ%s r%d", qd, nr);
			break;
		default:
			printf("S%02X%s r%d", c, qd, nr);
		}
	} else {
		switch (c & ~0x10) {
		case 0x2F:
			printf("SABM%s", pf ? "?" : "");
			break;
		case 0x43:
			printf("DISC%s", pf ? "?" : "");
			break;
		case 0x0F:
			printf("DM%s", pf ? "." : "");
			break;
		case 0x63:
			printf("UA%s", pf ? "." : "");
			break;
		case 0x87:
			printf("FRMR%s", pf ? "." : "");
			break;
		case 0x03:
			printf("UI%s", qd);
			break;
		default:
			printf("U%02X", c);
			break;
		}
	}
}

static int
print_unshifted_call(byte *p)
{
	int cc, i;

	for (cc = 6; cc > 1 && p[cc - 1] == ' '; --cc)
		;

	for (i = 0; i < cc; ++i)
		printf("%c", chr(p[i]));

	return (cc);
}

static int
print_shifted_call(byte *p)
{
	int cc, i;

	for (cc = 6; cc > 1 && (p[cc - 1] >> 1) == ' '; --cc)
		;

	for (i = 0; i < cc; ++i)
		printf("%c", chr(p[i] >> 1));

	return (cc);
}

static int
print_address(byte *p)
{
	int cc, ssid = (p[6] >> 1) & 15;

	cc = print_shifted_call(p);

	if (ssid)
		cc += printf("-%d", ssid);

	return (cc);
}

static void
print_netrom(byte *data, int len)
{
	if (len >= 1 + 6 && *data == 0xFF) {

		print_unshifted_call(data + 1);
		printf(" routes:");

		data += 1 + 6;
		len  -= 1 + 6;

		while (len >= 7 + 6 + 7 + 1) {
			printf(" ");
			print_address(data);
			printf("(");
			print_unshifted_call(data + 7);
			printf(") via ");
			print_address(data + 7 + 6);
			printf(" q%d", data[7 + 6 + 7]);

			data += 7 + 6 + 7 + 1;
			len  -= 7 + 6 + 7 + 1;
		}
		/* ought to be empty now, flow thru to hexdump */
	}
	if (len)
		printf("netrom ");

	for ( ; len > 0; --len, ++data) {
		if (*data > ' ' && *data < 0x7F)
			printf("%c", *data);
		else
			printf("\\%02X", *data);
	}
}

static void
print_packet(byte *packet, int paclen)
{
	byte *p;
	int n, col, cmdresp, pid;

	if (paclen < 7 + 7 + 1) {
		printf("# short packet, length %d", paclen);
		return;
	}

	cmdresp = ((packet[6] & 0x80) >> 6) | ((packet[13] & 0x80) >> 7);

	p = packet + 14;
	n = paclen - 14;          /* potential digis start from here */

	col = print_address(packet + 7);        /* SRC */

	while ((p[-1] & 1) == 0 && n > 7) {
		col += printf(">");
		col += print_address(p);             /* VIA */
		p += 7;
		n -= 7;
		if (p[-1] & 0x80)
			col += printf("*");              /* has been digipeated */
	}
	col += printf(">");
	col += print_address(packet + 0);        /* DST */
	col += printf(" ");

	while (col++ < 17)
		printf(" ");

	print_control(*p, cmdresp);
	p++;
	n--;

	if (n > 0) {
		pid = *p++;
		n--;

		printf(" ");

		switch (pid) {

		case 0xCF:
			print_netrom(p, n);
			break;

		default:
			printf("pid=%02X ", pid);

		case 0xF0:
			for ( ; n > 0; --n, ++p) {
				switch (*p) {
				case '\r':
					printf("\\r");
					break;
				case '\n':
					printf("\\n");
					break;
				case '\a':
					printf("\\a");
					break;
				default:
					printf("%c", chr(*p));
				}
			}
		}
	}
}

