
#include "standard.h"

/*
 * pcf8583 device address A0 or A2
 *
 * 00 control/status
 *     01 timer flag
 *     02 alarm flag
 *     04 alarm enable
 *     08 mask flag
 *     30 function:
 *          0 32kHz_clock
 *          1 50Hz_clock
 *          2 event_counter         **************
 *          3 test_modes
 *     40 hold last count flag
 *     80 stop counting flag        *****
 * 01
 *     0F digit 0, lsdigit
 *     F0 digit 1
 * 02
 *     0F digit 2
 *     F0 digit 3
 * 03
 *     0F digit 4
 *     F0 digit 5, msdigit
 *  ---/---
 * 07
 *     0F T0 "timer" digits
 *     F0 T1
 *
 * 08 alarm control
 *     07 Tx function:
 *          0 none
 *          1 ones
 *          2 hundreds
 *          3 ten thousands
 *          4 millions            ***************
 *          5 invalid
 *          6 invalid
 *          7 test mode
 *     08 timer int enable
 *     30 clock alarm function
 *          0 no event alarm
 *          1 event alarm
 *          2 invalid
 *          3 invalid
 *     40 timer alarm enable
 *     80 alarm int enable
 */
void
pcf8583_init(word dev)
{
	byte cmd[1 + 9];              /* address 0x00 and 9 bytes */

	bzero(cmd, sizeof(cmd));      /* address, count and Tx, all zero */

	cmd[1 + 0] = 0x20 | 0x80;     /* counter mode, but stopped */
	cmd[1 + 8] = 0x04;            /* Tx counts to 99 million */

	i2c_wr(dev, cmd, 1 + 9);

	cmd[1 + 0] = 0x20;            /* start counting */

	i2c_wr(dev, cmd, 1 + 1);
}

bool
pcf8583_count(word dev, dword *cnt)
{
	byte addr = 0x01, bcd[3];     /* bcd[7] would reach to Tx digits */

	if (i2c_wr(dev | NOSTOP, &addr, 1) && i2c_rd(dev, bcd, 3)) {

		*cnt = unpack_bcd_le(bcd, 3);
		return (TRUE);
	}
	return (FALSE);
}

